/****************************************************
**
** PICProgrammer.CPP
**
****************************************************/

//#include	"PICProgrammer.H"
#include	"PICP.H"
//#include	"PICProgrammerTiming.H"
#include	"PICPT.H"

#include	<stdio.h>
#include	<ctype.h>


#define HEX_PROGRAM_BASE   	0x0
#define HEX_ID_BASE   		0x2000
#define HEX_CONFIG_BASE   	0x2007
#define HEX_DATA_BASE   	0x2100


typedef enum {
	COMMAND_LOAD_CONFIG			= 0,
	COMMAND_LOAD_PROGRAM		= 2,
	COMMAND_LOAD_DATA			= 3,
	COMMAND_READ_PROGRAM		= 4,
	COMMAND_READ_DATA			= 5,
	COMMAND_INCREMENT			= 6,
	COMMAND_BEGIN_PROGRAM		= 8,
	COMMAND_ERASE_PROGRAM		= 9,
	COMMAND_ERASE_DATA			= 11
} Commands;

/*
**
*/
PICProgrammerClass::PICProgrammerClass( void ) :
	Hardware( 0 )
{
	Clear_Memory();
}

/*
**
*/
PICProgrammerClass::~PICProgrammerClass( void )
{
}

/*
**
*/
void PICProgrammerClass::Erase( void )
{
	Delay_Ms( 50 );
	Hardware->Program();
	Delay_Ms( 50 );

	Send_Command( COMMAND_LOAD_CONFIG );
	Send_Data( 0x3FFF );
	Send_Command( COMMAND_INCREMENT );
	Send_Command( COMMAND_INCREMENT );
	Send_Command( COMMAND_INCREMENT );
	Send_Command( COMMAND_INCREMENT );
	Send_Command( COMMAND_INCREMENT );
	Send_Command( COMMAND_INCREMENT );
	Send_Command( COMMAND_INCREMENT );
	Send_Command( 1 );
	Send_Command( 7 );
	Send_Command( COMMAND_BEGIN_PROGRAM );
	Delay_Ms( 10 );
	Send_Command( 1 );
	Send_Command( 7 );

	Send_Command( COMMAND_ERASE_PROGRAM );
	Send_Data( 0x3FFF );
	Send_Command( COMMAND_BEGIN_PROGRAM );
	Delay_Ms( 10 );

	Send_Command( COMMAND_ERASE_DATA );
	Send_Data( 0x3FFF );
	Send_Command( COMMAND_BEGIN_PROGRAM );
	Delay_Ms( 10 );

	Hardware->Off();
}

void	delay_short( int calls ) {
	if (calls > 0) {
		delay_short( calls-1 );
		delay_short( calls-1 );
		delay_short( calls-1 );
	}
}

/*
**
*/
int PICProgrammerClass::Read_Program_Verify( int mode, unsigned short *data, int length, short data_mask, int read_command, int load_command, int config, int config_offset ) 
{
	int index;

	Delay_Ms( 50 );
	Hardware->Program();
	Delay_Ms( 50 );

	if ( config ) {
		Send_Command( COMMAND_LOAD_CONFIG );
		Send_Data( 0x3FFF );
		for ( index = 0; index < config_offset; index++ ) {
			Send_Command( COMMAND_INCREMENT );
  		}
	}

	for ( index = 0; index < length; index++ ) {
		Send_Command( read_command );

	delay_short( 10 );

		int read_data = Get_Data() & data_mask;
		data[ index ] &= data_mask;

		if ( mode == READ_MODE ) {
			printf("\r%d   ", index );
			data[ index ] = read_data;
		} else {

			if ( (mode == PROGRAM_MODE) && (data[ index ] != read_data) ) {
				Send_Command( load_command );
				Send_Data( data[ index ] );
				Send_Command( COMMAND_BEGIN_PROGRAM );
			delay_short( 10 );
	//			Delay_Ms( 10 );
				Send_Command( read_command );
				read_data = Get_Data() & data_mask;
			}

			printf("\r%d   ", index );

			if ( data[ index ] != read_data ) {
				printf( "Verify Error index %d wanted %x read %x\n", index, data[index], read_data );
				return VERIFY_ERROR;
			} 
		} 
		Send_Command( COMMAND_INCREMENT );
	}

	printf( "\r" );

	Hardware->Off();

	return NO_ERROR;
}

/*
**
*/
int PICProgrammerClass::Read_Program_Verify_All( int mode ) 
{
	int result;
	result = Read_Program_Verify( mode, ProgramMemory, PROGRAM_MEMORY_SIZE, 0x3FFF, 
									COMMAND_READ_PROGRAM, COMMAND_LOAD_PROGRAM, 0, 0 );
	if (!result)
		result = Read_Program_Verify( mode, DataMemory, DATA_MEMORY_SIZE, 0xFF, 
									COMMAND_READ_DATA, COMMAND_LOAD_DATA, 0, 0 );
	if (!result)
		result = Read_Program_Verify( mode, ID, ID_MEMORY_SIZE, 0x3FFF, 
									COMMAND_READ_PROGRAM, COMMAND_LOAD_PROGRAM, 1, 0 );
	if (!result)
		result = Read_Program_Verify( mode, &Config, 1, 0x3FFF, 
									COMMAND_READ_PROGRAM, COMMAND_LOAD_PROGRAM, 1, 7 );
	return result;
}

/*
**
*/
int PICProgrammerClass::Read( void )
{
	int result = Read_Program_Verify_All( READ_MODE );
	if (!result) {
		printf( "Read Successful\n" );
	}
	return result;
}

/*
**
*/
int PICProgrammerClass::Verify( void )
{
	int result = Read_Program_Verify_All( VERIFY_MODE );
	if (result) {
		printf( "Verify Failed\n" );
	} else {
		printf( "Verify Successful\n" );
	}
	return result;
}

/*
**
*/
int PICProgrammerClass::Program( void )
{
	int result = Read_Program_Verify_All( PROGRAM_MODE );
	if (result) {
		printf( "Program Failed\n" );
	} else {
		printf( "Program Successful\n" );
	}
	return result;
}

/*
**
*/
void PICProgrammerClass::Clear_Memory( void )
{
	int index;
	for ( index = 0; index < PROGRAM_MEMORY_SIZE; index++ ) {
		ProgramMemory[ index ] = MEMORY_CLEAR;
	}
	for ( index = 0; index < DATA_MEMORY_SIZE; index++ ) {
		DataMemory[ index ] = MEMORY_CLEAR;
	}
	for ( index = 0; index < ID_MEMORY_SIZE; index++ ) {
		ID[ index ] = MEMORY_CLEAR;
	}
	Config = MEMORY_CLEAR;
	printf( "Memory Cleared\n" );
}

/*
**
*/
int PICProgrammerClass::Read_Hex_Nibble( FILE * file )
{
	char ch = toupper( fgetc( file ) );
//	printf( "Read Nibble %c\n", ch );
	if ( (ch >= '0') && (ch <= '9') )	
		return ch - '0';
	if ( (ch >= 'A') && (ch <= 'F') )	
		return ch - 'A' + 10;
	printf( "Hex Read Error\n" );
	return -1;
}

int PICProgrammerClass::Read_Hex_Byte( FILE * file )
{
	int result;
	result = Read_Hex_Nibble( file ) << 4;
	result |= Read_Hex_Nibble( file ) << 0;
	Checksum = (Checksum + result) & 0xFF;
//	printf( "Read Byte %x\n", result );
	return result;
}

int PICProgrammerClass::Read_Hex_Word( FILE * file )
{
	int result;
	result = Read_Hex_Byte( file );
	result |= Read_Hex_Byte( file ) << 8;
//	printf( "Read Word %x\n", result );
	return result;
}

/*
**
*/
void PICProgrammerClass::Load_Memory( const char * filename )
{
	Clear_Memory();

	FILE *in = fopen( filename, "rt" );
	if (in == NULL) {
		printf( "Failed to open %s\n", filename );	
		return;
	}

	while ( 1 ) 
	{
		while ( !feof(in) && (fgetc( in ) != ':') );
		
		if ( feof(in) ) {
			printf( "EOF Error\n" );	
			return;
		}

		Checksum = 0;

		int count = Read_Hex_Byte( in );
		int address = Read_Hex_Word( in );
		int type = Read_Hex_Byte( in );

		if ( type == 1 ) {
			printf( "Load Successful\n" );	
			return;
		}

		address = ((address >> 8) & 0xFF) + ((address << 8) & 0xFF00);
		address /= 2;
		count /= 2;

		if ((count < 0) || (address < 0) || (type < 0) ) {
			printf( "Read Failed\n" );	
			return;
		}

//		printf( "Address %x Count %x Type %x\n", address, count, type );

		while ( count-- ) {
			int data = Read_Hex_Word( in );

			if ( ( address >= HEX_PROGRAM_BASE ) && 
				 ( address <= HEX_PROGRAM_BASE + PROGRAM_MEMORY_SIZE ) ) {
				ProgramMemory[ address - HEX_PROGRAM_BASE ] = data;
			} else if ( ( address >= HEX_DATA_BASE ) && 
						 ( address <= HEX_DATA_BASE + DATA_MEMORY_SIZE ) ) {
				DataMemory[ address - HEX_DATA_BASE ] = data;
			} else if ( ( address >= HEX_ID_BASE ) && 
						 ( address <= HEX_ID_BASE + ID_MEMORY_SIZE ) ) {
				ID[ address - HEX_ID_BASE ] = data;
				
			} else if ( address == HEX_CONFIG_BASE ) {
				Config = data;
			} else {
				printf( "Unused data in hex file\n" );
			}

			address++;
		}

		Read_Hex_Byte( in );
		if ( Checksum ) {
			printf( "Checksum Error %x\n", Checksum );
			return;
		}
	}
}

/*
**
*/
void PICProgrammerClass::Save_Memory( const char * filename )
{
	if (filename) {}
}

void PICProgrammerClass::Dump_Memory( void )
{
	printf( "Memory Dump\n" );

	int index;
	printf( "Program:\n" );
	for ( index = 0; index < PROGRAM_MEMORY_SIZE; index++ ) {
		printf( "%04X ", ProgramMemory[ index ] );
	}

	printf( "\nData:\n" );
	for ( index = 0; index < DATA_MEMORY_SIZE; index++ ) {
		printf( "%02X  ", (unsigned char)DataMemory[ index ] );
	}
	printf( "\nID:" );
	for ( index = 0; index < ID_MEMORY_SIZE; index++ ) {
		printf( "%02X ", ID[ index ] );
	}
	printf( "\nConfig:" );
	printf( "0x%4X ", Config );
	printf( "\n" );

}

/*
**
*/
void PICProgrammerClass::Send_Command( int command )
{
	for ( int i = 0; i < COMMAND_SIZE; i++ ) {
		Hardware->Send_Data_Bit( command & 1 );
		command >>= 1;
	}
}

/*
**
*/
void PICProgrammerClass::Send_Data( int data )
{
	Hardware->Send_Data_Bit( 0 );
	for ( int i = 0; i < DATA_SIZE; i++ ) {
		Hardware->Send_Data_Bit( data & 1 );
		data >>= 1;
	}
	Hardware->Send_Data_Bit( 0 );
}

/*
**
*/
int PICProgrammerClass::Get_Data( void )
{
	int data = 0;
	Hardware->Get_Data_Bit();
	for ( int i = 0; i < DATA_SIZE; i++ ) {
		data >>= 1;
		data |= Hardware->Get_Data_Bit() ? 0x2000 : 0;
	}
	Hardware->Get_Data_Bit();
	return data;
}

