/****************************************************
**
** PICProgrammer.H
**
****************************************************/

#ifndef PICPROGRAMMER_H
#define PICPROGRAMMER_H

//#include	"PICProgrammerHardware.H"
#include	"PICPH.H"

#include	<stdio.h>

/*
**
*/
class PICProgrammerClass {

public:
	PICProgrammerClass( void );
	~PICProgrammerClass( void );

	void Set_Hardware( PICProgrammerHardwareClass *hardware ) { Hardware = hardware; }

	/*
	** PIC programming functions
	*/
	int Read_Program_Verify( int mode, unsigned short *data, int length, short data_mask, int read_command, int load_command, int config, int config_offset );
	int Read_Program_Verify_All( int mode );

	void Erase( void );
	int Program( void );
	int Read( void );
	int Verify( void );

	/* 
	** Memory functions
	*/
	void Clear_Memory( void );
	void Load_Memory( const char * filename );
	void Save_Memory( const char * filename );
	void Dump_Memory( void );

	int Read_Hex_Nibble( FILE * file );
	int Read_Hex_Byte( FILE * file );
	int Read_Hex_Word( FILE * file );

private:
	typedef enum {
		READ_MODE,
		PROGRAM_MODE,
		VERIFY_MODE
	} Modes;

	typedef enum {
		NO_ERROR,
		VERIFY_ERROR,
	} Errors;

	typedef enum {
		PROGRAM_MEMORY_SIZE	=	1024,
		DATA_MEMORY_SIZE	=	32, /*64,*/
		ID_MEMORY_SIZE		=	4,
		MEMORY_CLEAR		= 	0x3FFF,
		COMMAND_SIZE		= 	6,
		DATA_SIZE			= 	14,
	} ClassConstants;

	void Send_Command( int command );
	void Send_Data( int data );
	int Get_Data( void );

	PICProgrammerHardwareClass	*Hardware;

	unsigned short ProgramMemory[ PROGRAM_MEMORY_SIZE ];
	unsigned short DataMemory[ DATA_MEMORY_SIZE ];
	unsigned short ID[ ID_MEMORY_SIZE ];
	unsigned short Config;

	int	Checksum;
};

#endif // PICPROGRAMMER_H
