/****************************************************
**
** PICProgrammerHardwareParallel.CPP
**
****************************************************/

//#include "PICProgrammerHardwareParallel.H"
#include "PICPHP.H"
#include "PICPT.H"
#include <conio.h>
#include <assert.h>
#include <stdio.h>

#define	LPT_PORT_TABLE		((unsigned short *)0x00000408)

/*
**
*/
PICProgrammerHardwareParallelClass::PICProgrammerHardwareParallelClass( int lpt_port ) :
	PortRegister( 0 ),
	PortRegisterBits( 0 ),
	LPTPort( 0 )
{
	if ( lpt_port < 1 ) {	// search for the first lpt port with a programmer
		for ( LPTPort = 1; LPTPort <= 3; LPTPort++ ) {
			if ( Set_LPT_Port( LPTPort ) == NO_ERROR ) {
				break;
			}
		}
	} else {
		LPTPort = lpt_port;
		Set_LPT_Port( LPTPort );
	}
	if ( PortRegister ) {
	   PortRegisterBits = inportb( PortRegister );
	}
}

/*
**
*/
PICProgrammerHardwareParallelClass::~PICProgrammerHardwareParallelClass( void )
{
	if ( PortRegister ) {
		Set_State( VDD_OFF | VPP_OFF | CLOCK_OFF | DATA_OFF );
	}
}

/*
**
*/
int PICProgrammerHardwareParallelClass::Set_LPT_Port( int port )
{
	PortRegister = LPT_PORT_TABLE[ port-1 ];

	if ( (PortRegister != 0x3BC) && (PortRegister != 0x378) && (PortRegister != 0x278) ) {
		PortRegister = 0;
		return PORT_NOT_FOUND_ERROR;
	}

//	Delay_Ms( 0 );						// dummy call to setup CTC
	outportb( PortRegister + 1, 0xF);   // clear high nibble of LPT control reg
	
	Set_State( DATA_ON );
	Delay_Ms( 100 );
	if ( Get_Input() == 0 ) {
		PortRegister = 0;
		return PROGRAMMER_NOT_FOUND_ERROR;
	}
	Set_State( DATA_OFF );
	Delay_Ms( 100 );
	if ( Get_Input() != 0 ) {
		PortRegister = 0;
		return PROGRAMMER_NOT_FOUND_ERROR;
	}

	return NO_ERROR;
}

/*
**
*/
int	PICProgrammerHardwareParallelClass::Found( void )
{
	return (PortRegister != 0);
}

/*
**
*/
char * PICProgrammerHardwareParallelClass::Get_Description( void )
{
	static char Description[12];
	sprintf( Description, "LPT%d 0x%3X", LPTPort, PortRegister );
	return Description;
}

/*
**
*/
void PICProgrammerHardwareParallelClass::Set_State( int state_bits )
{
	assert( PortRegister != 0 );

	if ( state_bits & VPP_ON )				PortRegisterBits |= 8;
	if ( state_bits & VPP_OFF )			PortRegisterBits &= ~8;
	if ( state_bits & VDD_ON )				PortRegisterBits |= 4;
	if ( state_bits & VDD_OFF )			PortRegisterBits &= ~4;
	if ( state_bits & CLOCK_ON )			PortRegisterBits |= 2;
	if ( state_bits & CLOCK_OFF )			PortRegisterBits &= ~2;
	if ( state_bits & DATA_ON )			PortRegisterBits |= 1;
	if ( state_bits & DATA_OFF )			PortRegisterBits &= ~1;

	outportb( PortRegister, PortRegisterBits );
}

/*
**
*/
int PICProgrammerHardwareParallelClass::Get_Input( void )
{
	assert( PortRegister != 0 );
	return (inportb( PortRegister + 1 ) & 64) ? 1 : 0;
}

